@@ -45,6 +45,8 @@ UNINSTALL_LIBS_ENDPOINT = ('POST', 'api/2.0/libraries/uninstall')
 
 LIST_JOBS_ENDPOINT = ('GET', 'api/2.1/jobs/list')
 
+WORKSPACE_GET_STATUS_ENDPOINT = ('GET', 'api/2.0/workspace/get-status')
+
 RUN_LIFE_CYCLE_STATES = ['PENDING', 'RUNNING', 'TERMINATING', 'TERMINATED', 'SKIPPED', 'INTERNAL_ERROR']
 
 
@@ -328,3 +330,25 @@ class DatabricksHook(BaseDatabricksHook):
         :param json: json dictionary containing cluster_id and an array of library
         """
         self._do_api_call(UNINSTALL_LIBS_ENDPOINT, json)
+
+    def update_repo(self, repo_id: str, json: Dict[str, Any]) -> dict:
+        """
+
+        :param repo_id:
+        :param json:
+        :return:
+        """
+        repos_endpoint = ('PATCH', f'api/2.0/repos/{repo_id}')
+        return self._do_api_call(repos_endpoint, json)
+
+    def get_repo_by_path(self, path: str) -> Optional[str]:
+        """
+
+        :param path:
+        :return:
+        """
+        result = self._do_api_call(WORKSPACE_GET_STATUS_ENDPOINT, {'path': path})
+        if result.get('object_type', '') == 'REPO':
+            return str(result['object_id'])
+
+        return None
